<?php

namespace Drupal\group_bulk_operations\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Session\AccountInterface;

/**
 * Provides a group multiple role assignment configure form.
 */
class AssignGroupRoleMultiple extends ConfirmFormBase {

  /**
   * The array of groups to relate the user to.
   *
   * @var string[][]
   */
  protected $groupInfo = [];

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The group storage.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypemanager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Constructs a DeleteMultiple form object.
   *
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   Current user.
   */
  public function __construct(PrivateTempStoreFactory $temp_store_factory, EntityTypeManagerInterface $entity_type_manager, AccountInterface $current_user) {
    $this->tempStoreFactory = $temp_store_factory;
    $this->entityTypemanager = $entity_type_manager;
    $this->storage = $this->entityTypemanager->getStorage('group');
    $this->user = $this->entityTypemanager->getStorage('user');
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('tempstore.private'),
      $container->get('entity_type.manager'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'group_bulk_operations_multiple_group_assign_role_configure';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t("Please configure the user and role");
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('entity.group.collection');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Assign roles');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $this->groupInfo = $this->tempStoreFactory->get('group_bulk_operations_multiple_group_assign_role_configure')->get($this->currentUser->id());
    if (empty($this->groupInfo)) {
      return new RedirectResponse($this->getCancelUrl()->setAbsolute()->toString());
    }
    /** @var \Drupal\group\Entity\GroupInterface[] $nodes */
    $groups = $this->storage->loadMultiple(array_keys($this->groupInfo));

    $items = [];
    foreach ($this->groupInfo as $id => $langcodes) {
      foreach ($langcodes as $langcode) {
        $group = $groups[$id]->getTranslation($langcode);
        $key = $id . ':' . $langcode;
        $items[$key] = $group->label();
      }
    }

    $form['user'] = [
      '#type' => 'entity_autocomplete',
      '#target_type' => 'user',
      '#title' => $this->t('User'),
      '#required' => TRUE,
    ];
    $group_roles = $this->entityTypemanager
      ->getStorage('group_role')
      ->loadMultiple();
    $group_roles_options = [];
    foreach ($group_roles as $group_role) {
      if (in_array($group_role->get('audience'), ['anonymous', 'outsider'])) {
        continue;
      }
      $group_roles_options[$group_role->id()] = '['
        . $group_role->getGroupType()->label() . '] ' . $group_role->label();
    }
    $form['group_roles'] = [
      '#type' => 'checkboxes',
      '#title' => 'Group Roles',
      '#options' => $group_roles_options,
      '#required' => TRUE,
    ];
    $form['groups'] = [
      '#title' => $this->t('Groups'),
      '#theme' => 'item_list',
      '#items' => $items,
    ];
    $form = parent::buildForm($form, $form_state);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getValue('confirm') && !empty($this->groupInfo)) {
      $groups = $this->storage->loadMultiple(array_keys($this->groupInfo));

      $user = $this->user->load($form_state->getValue('user'));
      $roles = [];

      foreach ($form_state->getValue('group_roles') as $role) {
        if ($role) {
          $roles[] = ['target_id' => $role];
        }
      }

      if ($groups) {
        // Batch operations setting.
        $operations = [];
        foreach ($groups as $group) {
          $operations[] = [
            'group_bulk_operations_batch_add_member',
            [
              $user,
              $group,
              $roles,
            ],
          ];
        }
        $batch = [
          'title' => $this->t('Adding @user to Groups', ['@user' => $user->getDisplayName()]),
          'operations' => $operations,
          'finished' => 'group_bulk_operations_batch_add_member_finished',
        ];

        batch_set($batch);
      }

      $this->tempStoreFactory->get('group_bulk_operations_multiple_group_assign_role_configure')->delete($this->currentUser->id());
    }
    $form_state->setRedirect('entity.group.collection');
  }

}
